from langport.data.conversation import (
    ConversationSettings,
    SeparatorStyle,
)

# llama_cpp default template
llama_cpp = ConversationSettings(
    name="llama_cpp",
    roles=("User", "Assistant"),
    sep_style=SeparatorStyle.ADD_COLON_SINGLE,
    sep="\n",
)



from langport.data.conversation import ConversationHistory
from langport.data.conversation.conversation_settings import get_conv_settings
from langport.model.model_adapter import BaseAdapter


class LlamaCppAdapter(BaseAdapter):
    """The model adapter for LlamaCpp"""

    def match(self, model_path: str):
        return "ggml" in model_path

    def get_default_conv_template(self, model_path: str) -> ConversationHistory:
        settings = get_conv_settings("llama_cpp")
        return ConversationHistory(
            system=f"""Consider a conversation between User (a human) and Assistant (named Buddy).
Buddy is an INTP-T, a friendly, intelligent and multilingual AI assistant, based on LLaMA Transformers architecture, by OpenBuddy team on GitHub.
Buddy cannot access the Internet.
Buddy can fluently speak the user's language (e.g. English, Chinese).
Buddy can generate poems, stories, code, essays, songs, parodies, and more.
Buddy possesses vast knowledge about the world, history, and culture.
Buddy's responses are always safe, creative, high-quality, human-like, and interesting.
Buddy strictly refuses to discuss political, NSFW, illegal, abusive, offensive, or other sensitive topics.

User: Hi.
Assistant: Hi, I'm Buddy, your AI assistant. How can I help you today?""",
            messages=[],
            offset=0,
            settings=settings,
        )





from typing import Optional
from llama_cpp import Llama
from langport.model.executor.base import LocalModelExecutor


class LlamaCppExecutor(LocalModelExecutor):
    def __init__(
        self,
        model_name: str,
        model_path: str,
        device: str,
        num_gpus: int,
        max_gpu_memory: Optional[str],
        load_8bit: bool = False,
        cpu_offloading: bool = False,
    ) -> None:
        super(LlamaCppExecutor, self).__init__(
            model_name = model_name,
            model_path = model_path,
            device = device,
            num_gpus = num_gpus,
            max_gpu_memory = max_gpu_memory,
            load_8bit = load_8bit,
            cpu_offloading = cpu_offloading,
        )
 
    def load_model(self, model_path: str, from_pretrained_kwargs: dict):
        model = Llama(model_path, **from_pretrained_kwargs)
        return model, model.tokenizer()
